"""
Publish a new version:
$ git tag X.Y.Z -m "Release X.Y.Z"
$ git push --tags
$ # edit setup.py for new version
$ pip install --upgrade twine wheel
$ python setup.py sdist bdist_wheel --universal
$ twine upload dist/*
"""
from setuptools import setup

with open("README.md", "r") as fh:
    long_description = fh.read()

VERSION = '1.0.1'
DOWNLOAD_URL = (
    'https://github.com/ruru4143/youtube-websearch' + VERSION
)

setup(
    name='youtube_websearch',
    packages=['youtube_websearch'],
    version=VERSION,
    url='https://github.com/ruru4143/youtube-websearch',
    license='GPLv3',
    author='ruru4143',
    author_email='pypi@ruru.pw',
    description='searches youtube for videos without using the official api',
    long_description=long_description,
    long_description_content_type="text/markdown",
    install_requires=["beautifulsoup4"],
    download_url=DOWNLOAD_URL,
    keywords=[
        'youtube', 'yt', 'scaping', 'webapi', 'webscraper', 'video',
        'get video', 'scraper'
    ],
    classifiers=[
        'Intended Audience :: Developers',
        'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Natural Language :: English',
    ],
)



import urllib.request
import json

from bs4 import BeautifulSoup

base = "https://www.youtube.com/results?q="


def _get_webpage(url, useragent='Mozilla/5.0 (X11; Linux x86_64; rv:85.0) Gecko/20100101 Firefox/85.0'):
    """ get soup of the html content of a url
    :param url: url of the webpage
    :return: soup of the content form url, if it failes -> False
    """

    try:
        req = urllib.request.Request(url=url, headers={
            'User-Agent': useragent})
        resp = urllib.request.urlopen(req)
    except urllib.error.HTTPError:
        return False
    except urllib.error.URLError:
        return False

    if resp.code != 200:
        return False
    else:
        return BeautifulSoup(resp.read())


def plain_search(search_query, useragent='Mozilla/5.0 (X11; Linux x86_64; rv:85.0) Gecko/20100101 Firefox/85.0'):
    """ search youtube and get the raw json data back

    :param search_query: search string
    :return: raw ytInitialData form youtube
    """
    url = base + search_query.replace(" ", "+")

    soup = _get_webpage(url, useragent)

    script_tags = soup.findAll("script")

    search_json_data = None
    for script in script_tags:
        if script.string and "var ytInitialData = " == script.string[:20]:
            search_json_data = json.loads(script.string[20:-1]) # -1 is for removing ;

    if not search_json_data:
        print("No data found")
        return None

    return search_json_data


def messy_video_search(search_query, useragent='Mozilla/5.0 (X11; Linux x86_64; rv:85.0) Gecko/20100101 Firefox/85.0'):
    """ search youtube and get the raw video data back

    :param search_query: search string
    :return: raw video data from ytInitialData
    """
    search_data = plain_search(search_query, useragent)

    videos = search_data['contents']['twoColumnSearchResultsRenderer']['primaryContents']
    videos = videos['sectionListRenderer']['contents'][0]['itemSectionRenderer']['contents']

    videos = filter(lambda v: 'videoRenderer' in v.keys(), videos)  # remove non videos
    videos = [video['videoRenderer'] for video in videos]

    return videos


def video_search(search_query, useragent='Mozilla/5.0 (X11; Linux x86_64; rv:85.0) Gecko/20100101 Firefox/85.0'):
    """ search youtube and get the cleaned up video data back

    :param search_query: search string
    :return: cleaned up video data
    """
    videos = messy_video_search(search_query, useragent)

    videos_cleaned = []
    for video in videos:
        videos_cleaned.append({
                            'videoId': video['videoId'],
                            'title': video['title']['runs'][0]['text'],
                            'publishedTimeText': video['publishedTimeText']['simpleText'],
                            'lengthText': video['lengthText']['accessibility']['accessibilityData']['label'],
                            'viewCountText': video['viewCountText']['simpleText'],
                           })
    return videos_cleaned
